const cheerio = require('cheerio');
const fetch = require('node-fetch');

const TYPE_MAP = {
  剧场版: 'movie',
  TV: 'tv',
  OVA: 'ova',
  WEB: 'web',
  其他: '',
};

function getType(id) {
  const api = `https://bgm.tv/subject/${id}`;
  return fetch(api)
    .then(res => res.text())
    .then(cheerio.load)
    .then($ => TYPE_MAP[$('#headerSubject .nameSingle small').text()]);
}

/**
 * 搜索，获取 Bangumi ID 和中文标题
 * @param  {Object} item 番剧数据对象
 * @return {Promise}
 */
function search(item) {
  // Bangumi 搜索 API 在有某些特殊字符的情况下直接返回 404
  const encoded = encodeURIComponent(
    item.title
      .replace(/[-.\\/*!'()]/g, ' ')
      .replace(/\s+/g, ' ')
      .trim(),
  );
  const api = `https://api.bgm.tv/search/subject/${encoded}?responseGroup=large`;
  return fetch(api, { headers: { Cookie: `chii_searchDateLine=${Date.now() / 1e3 | 0}` } })
    .then(res => res.json())
    .then(({ error, list }) => {
      if (error) {
        throw new Error(error);
      }
      return list
        .filter(li => li.type === 2)
        .filter(li => Math.abs(new Date(item.begin) - new Date(li.air_date)) < 8.64e7);
    });
}

module.exports = async (item) => {
  let results = [];
  try {
    results = await search(item);
    if (!results.length) {
      return {};
    }
    const result = results[0];
    return {
      titleTranslate: result.name_cn
        ? { 'zh-Hans': [result.name_cn.replace(/&amp;/g, '&')] }
        : {},
      type: await getType(result.id),
      sites: [{ site: 'bangumi', id: String(result.id) }],
    };
  } catch (e) {
    console.log(e);
  }
  return {};
};
